"""
figures built for crypto prices.
"""
import matplotlib.pyplot as plt
import matplotlib as mpl
from matplotlib.colors import LinearSegmentedColormap, Normalize
import matplotlib.dates as mdates
from matplotlib.cm import ScalarMappable

import dateutil
import datetime
from dateutil.relativedelta import relativedelta

from surfingcrypto.ts import TS
import surfingcrypto.reporting.plotting as scplot
from surfingcrypto.reporting.trend_line import trend_line


##GLOBAL VARIABLES (?!?) TO SET PLOT STYLE
plt.style.use("dark_background")
mpl.rcParams["font.size"] = 4


class BaseFigure:
    """
    This is the base class object for all figures.

    Arguments:
        ts (:class:`surfingcrypto.ts.TS`) : `surfingcrypto.ts.TS` object
        graphstart (str) : date string in d-m-Y format (or relative from today eg. 1 month: `1m`,3 month: `3m`) from which to start the graph.
    """

    def __init__(
        self, ts, graphstart="1-1-2021",
    ):

        self.ts = ts

        if graphstart.lower() == "3m":
            self.graphstart = datetime.date.today() + relativedelta(months=-3)
        elif graphstart.lower() == "6m":
            self.graphstart = datetime.date.today() + relativedelta(months=-6)
        elif graphstart.lower() == "1m":
            self.graphstart = datetime.date.today() + relativedelta(months=-1)
        elif graphstart.lower() == "1y":
            self.graphstart = datetime.date.today() + relativedelta(years=-1)
        else:
            self.graphstart = dateutil.parser.parse(graphstart)

    def save(self, path):
        """
        save fig to specified path.

        Arguments:
            path (str) : path to output file.
        """
        return self.f.savefig(path)

    def center_series(self, ax, on="Close"):
        """
        centers the active series in the graph.
        This is useful when dealing only with "zoomed-in" views.
        
        Arguments:
            ax (:obj:`matplotlib.axes.Axes`) object is 
            on (str): `Close` or TA indicator name
        """
        xlim = mdates.num2date(ax.get_xlim())
        if on.lower() == "close":
            max = self.ts.df.loc[xlim[0] : xlim[1], "Close"].max()
            min = self.ts.df.loc[xlim[0] : xlim[1], "Close"].min()
        elif on.lower() == "macd":
            pass
        else:
            raise ValueError("Error: `on` parameter not known.")
        ax.set_ylim((min - 0.1 * min, max + 0.1 * max))

    def set_axes(self, xlims):
        """
        set all axes to the specified xlims.

        Arguments:
            xlims(:obj:`tuple`): tuple of xlims as datetime objects.
        """
        if hasattr(self, "axes"):
            for iax in self.axes:
                iax.grid(which="major", axis="x", linewidth=0.1)
                iax.grid(which="major", axis="y", linewidth=0.05)
                iax.set_xlim(xlims)
                iax.yaxis.set_label_position("left")
                iax.yaxis.tick_left()
        elif hasattr(self, "ax"):
            self.ax.grid(which="major", axis="x", linewidth=0.1)
            self.ax.grid(which="major", axis="y", linewidth=0.05)
            self.ax.set_xlim(xlims)
            self.ax.yaxis.set_label_position("left")
            self.ax.yaxis.tick_left()
        else:
            raise NotImplementedError


class SimplePlot(BaseFigure):
    """
    This is the basic price plot.
    Candlesticks + volume.

    Arguments:
        ts (:class:`surfingcrypto.ts.TS`) : `surfingcrypto.ts.TS` object
        graphstart (str) : date string in d-m-Y format 
            (or relative from today eg. 1 month: `1m`,3 month: `3m`) from which to start the graph.
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.default_plot()

    def default_plot(self):
        """
        default plotting.
        """
        # figure
        self.f, self.axes = plt.subplots(
            2, 1, sharex=True, gridspec_kw={"height_ratios": [4, 1]}, dpi=200,
        )
        # plot
        scplot.candlesticks(
            self.ts,
            ax=self.axes[0],
            volume=True,
            vol_ax=self.axes[1],
            style="candlesticks",
        )
        # axes look
        self.set_axes(
            (
                self.graphstart,
                self.ts.df.index[-1] + datetime.timedelta(days=5),
            )
        )
        self.axes[0].set_title(
            self.ts.coin, fontsize=10, va="center", ha="center", pad=20
        )
        self.center_series(self.axes[0], on="Close")
        # log
        print(f"{self.ts.coin} plotted.")


class TaPlot(BaseFigure):
    """
    this is the Technical Analysis plot.
    It shows (at current time): candlesticks,
    volume and 3 TA Indicators (MACD, BB bands and RSI)
    Can be easily modified to fit other and/or more indicators.

    Arguments:
        trendlines (bool) : UNDER DEVELOPEMENT! - plot also trendlines 
            calculated with `src.trend_line` class.
        ts (:class:`surfingcrypto.ts.TS`) : `surfingcrypto.ts.TS` object
        graphstart (str) : date string in d-m-Y format 
            (or relative from today eg. 1 month: `1m`,3 month: `3m`) from
            which to start the graph.
 
    """

    def __init__(self, trendlines=False, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.ta_plot(trendlines)

    def ta_plot(self, trendlines):
        """
        plotting function.
        """
        # figure
        self.f, self.axes = plt.subplots(
            5,
            1,
            sharex=True,
            gridspec_kw={"height_ratios": [2, 1, 1, 1, 1]},
            dpi=200,
            figsize=(7.5, 7.5),
        )
        # ta indicators
        self.ts.ta_indicators()
        # plots
        scplot.candlesticks(
            self.ts,
            ax=self.axes[0],
            volume=True,
            vol_ax=self.axes[1],
            style="candlesticks",
        )
        scplot.plot_moving_averages(self.ts, ax=self.axes[0])
        scplot.plot_macd(self.ts, self.axes[2], plot_lines=False)
        scplot.candlesticks(self.ts, self.axes[3], style="ohlc")
        scplot.plot_bb(self.ts, self.axes[3])
        scplot.plot_RSI(self.ts, self.axes[4])

        # trendlines
        if trendlines:
            pass
            # trend=trend_line(self,trendln_start="01-01-2021")
            # trend.build(compute=True,
            #     method="NCUBED",
            #     window=125,
            #     save_output=False
            # )
            # trend.plot_trend(ax=self.axes[0],
            #     extend=False,
            #     nbest=2,
            #     show_min_maxs=False
            #     )
        # axes look
        self.set_axes(
            (
                self.graphstart,
                self.ts.df.index[-1] + datetime.timedelta(days=5),
            )
        )
        self.axes[0].set_title(
            self.ts.coin, fontsize=10, va="center", ha="center", pad=20
        )
        self.center_series(self.axes[0], on="Close")
        # log
        print(f"{self.ts.coin} plotted.")


class ATHPlot(BaseFigure):
    """
    distance from ATH plot.

    Arguments:
        ts (:class:`surfingcrypto.ts.TS`) : `surfingcrypto.ts.TS` object
        graphstart (str) : date string in d-m-Y format 
            (or relative from today eg. 1 month: `1m`,3 month: `3m`) from which to start the graph.

    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.plot()

    def plot(self):
        """
        plot distance from ath points.

        Note:
            ATM it its a zoomed-in view.
        """
        # figure
        self.f, self.ax = plt.subplots(dpi=200,)
        # compute distance
        self.ts.distance_from_ath()

        # normalizzato su tutto intervallo
        vmin = self.ts.df["distance_ATH"].min()
        vmax = self.ts.df["distance_ATH"].max()

        # cmap normalized and mappable
        norm = Normalize(vmin=vmin, vmax=vmax)
        cmap = LinearSegmentedColormap.from_list(
            "colorbar", ["green", "orange", "red", "magenta"]
        )
        colors = [
            mpl.colors.rgb2hex(x)
            for x in cmap(norm(self.ts.df["distance_ATH"]))
        ]
        cmappable = ScalarMappable(norm, cmap=cmap)

        # points
        self.ax.scatter(self.ts.df.index, self.ts.df.Close, c=colors, s=2)
        # colorbar
        self.f.colorbar(cmappable)

        # axes look
        self.set_axes(
            (
                self.graphstart,
                self.ts.df.index[-1] + datetime.timedelta(days=5),
            )
        )
        self.ax.set_title(
            "Distance from All Time High: " + self.ts.coin,
            fontsize=10,
            va="center",
            ha="center",
            pad=20,
        )
        # log
        print(f"{self.ts.coin} ATH plotted.")


class PortfolioPlot(BaseFigure):
    """
    Portfolio plots

    Arguments:
        portfolio (:class:`surfingcrypto.portfolio.portfolio`) : 
            `surfingcrypto.portfolio.portfolio` object
        graphstart (str) : date string in d-m-Y format 
            (or relative from today eg. 1 month: `1m`,3 month: `3m`) from which to start the graph.

    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.plot()



import pandas as pd
import trendln
import pickle
import datetime
import numpy as np
import matplotlib.pyplot as plt
import os
import dateutil


class trend_line:
    def __init__(self, ts, trendln_start="01-01-2021"):

        self.name = ts.coin
        self.df = ts.df
        self.trendln_start = dateutil.parser.parse(
            trendln_start, dayfirst=True
        )

    def build(
        self,
        compute=True,
        method=None,
        data_type=None,
        accuracy=None,
        window=125,
        save_output=False,
        execution_day=None,
    ):
        self.data_type = data_type
        self.accuracy = accuracy
        self.window = window

        if compute is False and execution_day is None:
            raise ValueError("Must specify day of execution to load.")

        if compute is False and method is None:
            raise ValueError("Must specify method of execution to load.")

        if method == None or method == "NSQUREDLOGN":
            self.method_name = "NSQUREDLOGN"
            self.method = trendln.METHOD_NSQUREDLOGN

        elif method == "PROBHOUGH":
            self.method_name = method
            self.method = trendln.METHOD_PROBHOUGH

        elif method == "NCUBED":
            self.method_name = method
            self.method = trendln.METHOD_NCUBED

        elif method == "HOUGHLINES":
            self.method_name = method
            self.method = trendln.METHOD_HOUGHLINES

        elif method == "HOUGHPOINTS":
            self.method_name = method
            self.method = trendln.METHOD_HOUGHPOINTS

        else:
            raise ValueError("Method not known")

        start_str = str(self.trendln_start.strftime("%d-%m-%Y"))[:10]
        self.proj = (
            "data/trend_lines/"
            + self.name
            + "_"
            + str(start_str)[:10]
            + "/"
            + self.method_name
            + "/"
        )
        picklename = self.proj + str(self.name) + "-trends" + "_"

        if save_output and not os.path.exists(self.proj):
            os.makedirs(self.proj)

        if compute:

            self.execution_day = datetime.date.today().strftime("%d-%m-%Y")
            self.compute()
            if save_output:
                with open(picklename + self.execution_day + ".pkl", "wb") as f:
                    pickle.dump([self.support, self.resistance], f)
        else:
            self.execution_day = execution_day
            with open(picklename + self.execution_day + ".pkl", "rb") as f:
                pkl = pickle.load(f)
                self.support = pkl[0]
                self.resistance = pkl[1]
        return

    def compute(self):

        if self.data_type is not None and self.data_type.lower() == "low-high":
            h = (
                self.df.Low[self.trendln_start :],
                self.df.High[self.trendln_start :],
            )
            self.skip_indexes = len(self.df.Low[: self.trendln_start]) - 1
        else:
            h = self.df.Close[self.trendln_start :]
            self.skip_indexes = len(self.df.Close[: self.trendln_start]) - 1

        self.support, self.resistance = trendln.calc_support_resistance(
            h, accuracy=self.accuracy, window=self.window, method=self.method,
        )
        ##SUPPORT
        # minimaIdxs - sorted list of indexes to the local minima
        # pmin - [slope, intercept] of average best fit line through all local minima points
        # mintrend - sorted list containing (points, result) for local minima trend lines
        # points - list of indexes to points in trend line
        # result - (slope, intercept, SSR, slopeErr, interceptErr, areaAvg)
        # slope - slope of best fit trend line
        # intercept - y-intercept of best fit trend line
        # SSR - sum of squares due to regression
        # slopeErr - standard error of slope
        # interceptErr - standard error of intercept
        # areaAvg - Reimann sum area of difference between best fit trend line
        #   and actual data points averaged per time unit
        # minwindows - list of windows each containing mintrend for that window
        # RESISTANCE
        # maximaIdxs - sorted list of indexes to the local maxima
        # pmax - [slope, intercept] of average best fit line through all local maxima points
        # maxtrend - sorted list containing (points, result) for local maxima trend lines
        # see for mintrend above
        # maxwindows - list of windows each containing maxtrend for that window

    def obtain_correct_indexes(self, list):
        return [x + self.skip_indexes for x in list]

    def extend(self, periods=None):
        if periods is None:
            periods = 10

        for col in self.df_trends.columns.tolist():
            self.df_trends.loc[
                self.df_trends[col].duplicated(keep="first"), col
            ] = np.nan

        ordinal_index = self.df_trends.index.map(
            datetime.datetime.toordinal
        ).to_numpy()

        ext = []
        for column in self.df_trends.columns:
            mask = ~np.isnan(ordinal_index) & ~np.isnan(self.df_trends[column])
            slope, intercept, r_value, p_value, std_err = linregress(
                ordinal_index[mask], self.df_trends[column][mask]
            )

            # firstday=self.df_trends[column].last_valid_index()
            firstday = self.df_trends[column].last_valid_index()

            firstday = firstday.toordinal()

            future = np.arange(firstday + 1, firstday + periods, 1)
            future = np.array(future)

            line = [slope * xi + intercept for xi in future]
            future = [datetime.datetime.fromordinal(i) for i in future]

            ext.append(pd.Series(line, index=future, name=column))

        ext = pd.concat(ext, axis=1)

        self.df_trends = pd.concat([self.df_trends, ext]).sort_index()
        self.df_trends = self.df_trends.groupby(self.df_trends.index).agg(
            nanmean
        )
        self.df_trends = self.df_trends.interpolate(
            how="index", limit_area="inside"
        )

    def plot_trend(self, ax, nbest=2, extend=True, show_min_maxs=False):

        if extend:
            raise ValueError("Extend not implemented")
            # self.extend(periods=days)

        for i in range(len(self.support[3])):
            if len(self.support[3][i]) != 0:
                if len(self.support[3][i]) < nbest:
                    nbest_sup = len(self.support[3])
                else:
                    nbest_sup = nbest

                for ii in range(nbest_sup):
                    self.df.iloc[
                        self.obtain_correct_indexes(self.support[3][i][ii][0])
                    ].plot(
                        y="Close",
                        ax=ax,
                        color="lightskyblue",
                        linestyle="--",
                        linewidth=0.5,
                        legend=False,
                        alpha=1,
                    )

        for i in range(len(self.resistance[3])):
            if len(self.resistance[3][i]) != 0:
                if len(self.resistance[3][i]) < nbest:
                    nbest_res = len(self.support[3])
                else:
                    nbest_res = nbest

                for ii in range(nbest_res):
                    self.df.iloc[
                        self.obtain_correct_indexes(
                            self.resistance[3][i][ii][0]
                        )
                    ].plot(
                        y="Close",
                        ax=ax,
                        color="magenta",
                        linestyle="--",
                        linewidth=0.5,
                        legend=False,
                        alpha=1,
                    )

        # show points that are used for trendline computation
        if show_min_maxs:
            # mins
            self.df.iloc[
                self.obtain_correct_indexes(self.support[0])
            ].reset_index().plot(
                ax=ax,
                x="Date",
                y="Close",
                kind="scatter",
                s=2,
                color="lightskyblue",
                zorder=10,
            )
            # maxs
            self.df.iloc[
                self.obtain_correct_indexes(self.resistance[0])
            ].reset_index().plot(
                ax=ax,
                x="Date",
                y="Close",
                kind="scatter",
                s=2,
                color="magenta",
                zorder=10,
            )

        # annotation
        self.lastday = self.df.index.tolist()[-1].strftime("%d-%m-%Y")
        s = (
            "Trend lines computed on: "
            + self.execution_day
            + "\n"
            + "Last price on: "
            + str(self.lastday)
        )
        ax.text(
            s=s,
            x=0.05,
            y=0.985,
            ha="left",
            color="white",
            va="top",
            transform=ax.transAxes,
            fontsize=4,
        )

        # iax.legend(loc="lower right",fontsize=5)

        return



"""
coinbase API client and custom objects.
"""
import json
import os
import pandas as pd
import datetime as dt

from coinbase.wallet.client import Client


class CB:

    """
    Interface to the Coinbase python API.

    Note:
        Requires an API Key and API Secret stored in `coinbase.json`.
        The permissions required to run this code are *read-only* and are the
        following.

        - ```wallet:sells:read```
        - ```wallet:accounts:read```
        - ```wallet:deposits:read```
        - ```wallet:orders:read```
        - ```wallet:trades:read```
        - ```wallet:transactions:read```
        - ```wallet:user:read```

    Arguments:
        configuration (:obj:`surfingcrypto.config.config`): package
            configuration object

    Attributes:
        configuration (:obj:`surfingcrypto.config.config`): package
            configuration object
        client (:obj:`coinbase.wallet.client.Client`): client object for
            making requests to coinbase API
        user (:obj:`coinbase API object`): user API object
    """

    def __init__(self, configuration):

        if hasattr(configuration, "coinbase"):
            self.configuration = configuration
            self.client = Client(
                configuration.coinbase["key"], configuration.coinbase["scrt"]
            )
            self.user = self.client.get_current_user()
        else:
            raise ValueError("config.json file must contain a coinbase token.")

    def _get_paginated_items(self, api_method, limit=100):
        """
        Generic getter for paginated items
        """
        all_items = []
        starting_after = None
        while True:
            items = api_method(limit=limit, starting_after=starting_after)
            if items.pagination.next_starting_after is not None:
                starting_after = items.pagination.next_starting_after
                all_items += items.data
            else:
                all_items += items.data
                break
        return all_items

    def _get_accounts(self, limit=100):
        """
        get all accounts through pagination.
        """
        return self._get_paginated_items(self.client.get_accounts, limit=100)

    def _get_transactions(self, account, limit=100):
        """
        get all transaction from specified account through pagination.

        Arguments
            account (:obj:`coinbase.wallet.model.ApiObject`) : coinbase
                account object.
        """
        return self._get_paginated_items(account.get_transactions, limit)

    def get_active_accounts(self):
        """
        get active accounts (balance > 0)

        Return:
            active (:obj:`list` of :obj:`coinbase.ApiObject`): list of
                coinbase accounts
        """
        active = []
        accounts = self._get_accounts()
        for account in accounts:
            if account["native_balance"]["amount"] != "0.00":
                active.append(account)
        return active

    def get_all_accounts_with_transactions(self, verbose=False):
        """
        get all accounts with recorded transactions.

        Note:
            SUPER FUCKING SLOW!!!

        Params:
            verbose (bool): verbose mode in order to check status

        Return:
            has_transactions (:obj:`list` of :obj:`coinbase.ApiObject`): list
                of coinbase accounts
            timeranges (:obj:`list` of :obj:`dict`): list of
                dictionaries with dates of first and last transaction
                for each account in `has_transactions`
        """
        has_transactions = []
        timeranges = []
        accounts = self._get_accounts()
        _list = len(accounts)
        for account, i in zip(accounts, range(_list)):
            if verbose:
                print(f"## {i+1} of {_list}")
                print(account["currency"])
            transactions = self._get_transactions(account)
            if len(transactions) > 0:
                timerange = {
                    0: transactions[0]["created_at"],
                    1: transactions[-1]["created_at"],
                }
                has_transactions.append(account)
                timeranges.append(timerange)
        return has_transactions, timeranges

    def get_accounts_from_list(self, _list=None):
        """
        gets accounts from a list dumped locally in the config folder.

        Return:
            from_list (:obj:`list` of :obj:`coinbase.ApiObject`): list
                of coinbase accounts
        """
        if isinstance(_list, list):
            from_list = []
            for account in _list:
                from_list.append(
                    self.client.get_account(account["account_id"])
                )
        else:
            raise ValueError("Must be a list.")
        return from_list


class MyCoinbase(CB):
    """
    This class is the user's Coinbase account.

    It stores all cryptocurrency accounts - either active or historic.

    It inherits from `surfingcrypto.coinbase.CB` the client methods
    and all information regarding the user.
    It automatically dumps temporary data in order to load accounts faster.

    Arguments:
        active_accounts (bool) : default `True`, select
            active (balance>0) accounts only.
        force (bool): force update from API even if
            local cache is found.
        configuration (:obj:`surfingcrypto.config.config`) : package
            configuration object.

    Attributes:
        accounts (:obj:`list` of :obj:`coibase.model.ApiObject`):
            list of selected accounts.
        active_accounts (list): list of string names of active accounts (balance>0.00)
        history (:obj:`surfingcrypto.portfolio.coinbase.TransactionsHistory`):
            `TransactionsHistory` object
        last_updated (:obj:`datetime.datetime`): datetime of
            last updates of account list.
        timeranges (:obj:`list` of :obj:`dict`): list of
            dictionaries with dates of first and last transaction for
            each account
        isHistoric (bool): if module has been loaded in historic mode,
            a.k.a. all accounts with a transaction in the record
        json_path (str): path to json dump file

    """

    def __init__(self, active_accounts=True, force=False, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.start(active_accounts, force)
        self._set_active_accounts()
        return

    def start(self, active_accounts, force):
        """
        start MyCoinbase module, loading accounts as specified.

        Arguments:
            active_accounts (bool): get only active (balance>0) accounts
            force (bool): force update from API even if local cache is found.
        """
        self.json_path = (
            self.configuration.config_folder + "/coinbase_accounts.json"
        )

        if active_accounts:
            self.accounts = self.get_active_accounts()
            self.isHistoric = False
            self.last_updated = dt.datetime.utcnow()
        elif active_accounts is False:
            self.isHistoric = True
            # if faile not found, fetch all instead of failing
            if os.path.isfile(self.json_path) and not force:
                accounts, last_updated = self._load_accounts()
                self.last_updated = dt.datetime.strptime(
                    last_updated, "%Y-%m-%dT%H:%M:%SZ"
                )
                if dt.datetime.utcnow() - self.last_updated < dt.timedelta(
                    days=7
                ):
                    self.accounts = self.get_accounts_from_list(accounts)
                else:
                    (
                        self.accounts,
                        self.timeranges,
                    ) = self.get_all_accounts_with_transactions()
                    self._dump_accounts()
            else:
                (
                    self.accounts,
                    self.timeranges,
                ) = self.get_all_accounts_with_transactions()
                self._dump_accounts()
                self.last_updated = dt.datetime.now(dt.timezone.utc)
        else:
            raise ValueError("Either true or false.")

    def _dump_accounts(self):
        """
        dumps fetched accounts for faster execution time in following sessions.
        """
        _list = []
        for account, timerange in zip(self.accounts, self.timeranges):
            _list.append(
                {
                    "currency": account["currency"],
                    "account_id": account["id"],
                    "balance": account["balance"]["amount"],
                    "timerange": timerange,
                }
            )
        dump = {
            "datetime": dt.datetime.utcnow().strftime("%Y-%m-%dT%H:%M:%SZ"),
            "accounts": _list,
        }
        with open(self.json_path, "w") as f:
            json.dump(dump, f, indent=4, default=str)

    def _load_accounts(self):
        """
        load accounts from dumped `coinbase_accounts.json` file.
        """
        with open(self.json_path, "rb") as f:
            dict = json.load(f)
            accounts = dict["accounts"]
            last_updated = dict["datetime"]
        return accounts, last_updated

    def _set_active_accounts(self):
        """
        sets the currently active accounts, a.k.a. balance >0
        this is required for the case when the module is loaded in historic mode, 
        so to distinguish active accounts from all known accounts.
        """
        active = []
        if hasattr(self, "accounts"):
            for account in self.accounts:
                if float(account.native_balance.amount) > 0:
                    active.append(account.currency)
        self.active_accounts = active

    def mycoinbase_report(self):
        """
        Nicely formatted report of accounts portfolio and user total
        balance in EUR.

        Returns:
            s (str): text
        """
        s = ""
        tot = 0
        if hasattr(self, "accounts"):
            for account in self.accounts:
                if float(account.native_balance.amount) > 0:
                    s = (
                        s
                        + str(account.currency)
                        + " : "
                        + str(account.native_balance)
                        + "\n"
                    )
                    tot += float(account.native_balance.amount)
            s = s + "---\n" + "Portfolio: EUR " + "{:.2f}".format(tot)
            return s
        else:
            raise ValueError("Must get accounts first.")

    def get_history(self):
        """
        start a TransactionsHistory object
        """
        self.history = TransactionsHistory(self)
        pass

    def __repr__(self):
        return (
            f"MyCoinbase( isHistoric:{self.isHistoric},"
            f" last_updated:{self.last_updated},"
            f" N_accounts:{len(self.accounts)})"
        )

    def __str__(self):
        return (
            f"MyCoinbase( isHistoric:{self.isHistoric},"
            f" last updated:{self.last_updated},"
            f" N_accounts:{len(self.accounts)})"
        )


class TransactionsHistory:
    """
    This objects fetches all coinbase transactions and parses them into a
    known format.

    Arguments:
        mycoinbase (:obj:`surfingcrypto.coinbase.coinbase.MyCoinbase`):
            :obj:`MyCoinbase` object.

    Attributes:
        df (:obj:`pandas.DataFrame`): dataframe of all known transactions
        transactions (:obj:`list` of :obj:`coibase.model.ApiObject`): list
            of processed transactions.
        known_types (list): list of string names
            of supported transaction types.
        unhandled_trans (:obj:`list` of :obj:`dict`): list informations of
            unhandled transactions.
        errors (:obj:`list` of :obj:`coibase.model.ApiObject`): list of 
            transactions that resulted in an error
        error_log (:obj:`list` of :obj:`dict`): list informations of
            transactions that resulted in an error.
    """

    def __init__(self, mycoinbase):
        self._mycoinbase = mycoinbase
        self.known_types = [
            "buy",
            "sell",
            "trade",  # verify
            "send",  # verify
            "fiat_withdrawal",
            "fiat_deposit",
        ]
        self._start()

    def _start(self):
        if self._mycoinbase.isHistoric is True:
            # ci sonotanti tipi di transactions
            self.unhandled_trans = []
            # error log for when failing handling known transactions types
            self.error_log = []
            self.errors = []

            self.df = []
            self.transactions = []

            if hasattr(self._mycoinbase, "accounts"):
                for account in self._mycoinbase.accounts:
                    self._handle_transactions(account)
                self.df = pd.DataFrame(self.df).set_index("datetime")
                self.df.sort_index(inplace=True)
                order = [
                    "type",
                    "amount",
                    "symbol",
                    "native_amount",
                    "nat_symbol",
                ]
                neworder = order + [
                    c for c in self.df.columns if c not in order
                ]
                self.df = self.df.reindex(columns=neworder)
            else:
                raise ValueError(
                    "MyCoinbase objects must have an accounts attribute."
                )
        else:
            raise ValueError("Must load historic data.")

    def _handle_transactions(self, account):
        """
        handles the transactions based on type.
        """
        for transaction in self._mycoinbase._get_transactions(account):
            if transaction["type"] in self.known_types:
                self._process_transaction(account, transaction)
            else:
                self.unhandled_trans.append(
                    {
                        "transaction_type": transaction["type"],
                        "account_id": account["id"],
                        "transaction_id": transaction["id"],
                    }
                )

    def _process_transaction(self, account, transaction):
        """
        process a transaction.
        """
        # spot price??
        symbol, amount, datetime = self._get_transact_info(transaction)
        nat_amount, nat_symbol = self._get_native_amount(transaction)
        try:
            total, subtotal, total_fee = self._get_transact_data(
                account, transaction
            )
            self.transactions.append(transaction)
        except Exception as e:
            total, subtotal, total_fee = None, None, None

            self.errors.append(transaction)
            self.error_log.append(
                {
                    "transaction_type": transaction["type"],
                    "account_id": account["id"],
                    "transaction_id": transaction["id"],
                    "info": {
                        "amount": amount,
                        "symbol": symbol,
                        "date": datetime,
                    },
                    "error_log": e,
                }
            )

        if subtotal is None:
            spot_price = nat_amount / amount
        else:
            spot_price = subtotal / amount

        if transaction["type"] == "trade":
            trade_id = transaction["trade"]["id"]
        else:
            trade_id = None

        self.df.append(
            {
                "type": transaction["type"],
                "datetime": datetime,
                "symbol": symbol,
                "amount": amount,
                "native_amount": nat_amount,
                "nat_symbol": nat_symbol,
                "total": total,
                "subtotal": subtotal,
                "total_fee": total_fee,
                "spot_price": abs(spot_price),
                "trade_id": trade_id,
            }
        )

    def _get_transact_info(self, transaction):
        """
        get basic info from transaction.
        """
        symbol = transaction["amount"]["currency"]
        amount = float(transaction["amount"]["amount"])
        datetime = transaction["created_at"]
        return symbol, amount, datetime

    def _get_native_amount(self, transaction):
        """
        gets native amount from transaction
        """
        native_amount = float(transaction["native_amount"]["amount"])
        native_symbol = transaction["native_amount"]["currency"]
        return native_amount, native_symbol

    def _get_transact_data(self, account, transaction):
        """
        gets required additional data (eg. fees)
        from different kinds of transactions.
        """
        if transaction["type"] == "sell":
            t = self._mycoinbase.client.get_sell(
                account["id"], transaction["sell"]["id"]
            )
        elif transaction["type"] == "buy":
            t = self._mycoinbase.client.get_buy(
                account["id"], transaction["buy"]["id"]
            )
        elif transaction["type"] == "fiat_withdrawal":
            t = self._mycoinbase.client.get_withdrawal(
                account["id"], transaction["fiat_withdrawal"]["id"]
            )
        elif transaction["type"] == "fiat_deposit":
            t = self._mycoinbase.client.get_deposit(
                account["id"], transaction["fiat_deposit"]["id"]
            )
        elif transaction["type"] in ["send", "trade"]:
            t = None
        else:
            raise ValueError("missing specification in source code.")

        if t is not None:
            if transaction["type"] in ["fiat_deposit", "fiat_withdrawal"]:
                # in these two transactions `total` is not present
                total = float(t["amount"]["amount"])
            else:
                total = float(t["total"]["amount"])
            subtotal = float(t["subtotal"]["amount"])
            total_fee = 0
            if "fees" in t:
                for fee in t["fees"]:
                    total_fee += float(fee["amount"]["amount"])
        else:
            total, subtotal, total_fee = None, None, None
        return total, subtotal, total_fee

    def __repr__(self):
        return (
            f"TransactionsHistory("
            f"Transactions:{len(self.transactions)+len(self.unhandled_trans)+len(self.error_log)} "
            f"- Processed:{len(self.df)}, "
            f"Unhandled:{len(self.unhandled_trans)} "
            f"- Errors:{len(self.error_log)}"
            ")"
        )

    def __str__(self):
        return (
            f"TransactionsHistory("
            f"Transactions:{len(self.transactions)+len(self.unhandled_trans)+len(self.error_log)} "
            f"- Processed:{len(self.df)}, "
            f"Unhandled:{len(self.unhandled_trans)} "
            f"- Errors:{len(self.error_log)}"
            ")"
        )

    def executed_without_errors(self) -> bool:
        """
        check if the object was executed with errors

        Returns:
            bool: execution resulted in errors
        """
        if len(self.unhandled_trans) + len(self.error_log) > 0:
            return False
        else:
            return True



"""
plotting methods.
"""
import mplfinance as mplf
import numpy as np
import matplotlib
import matplotlib.pyplot as plt

# warning di mplfinance per too many data in candlestick plot
import warnings

warnings.filterwarnings("ignore")


def candlesticks(ts, ax, volume=False, vol_ax=None, style="candlesticks"):
    """
    plotting candlesticks into a matplotlib.axes.Axes object.

    Args:
        ts (:class:`surfingcrypto.ts.TS`) : `surfingcrypto.ts.TS` object
        ax (:class:`matplotlib.axes.Axes`) : matplotlib ax to plot
            candlesticks into.
        volume (bool,optional): plot volume histogram data into a
            another ax specified with `vol_ax`
        vol_ax (:class:`matplotlib.axes.Axes`) : matplotlib ax
             to plot volume histogram into.
        style (str,optional): style of plotting candlesticks, `candlesticks`
            is default but `ohlc` style is used in BB bands plotting.
    """
    if style == "candlesticks":
        if volume is False:
            mplf.plot(
                ts.df,
                ax=ax,
                type="candle",
                style="yahoo",
                show_nontrading=True,
                loc="lower left",
            )
        elif volume and vol_ax is None:
            raise ValueError("Must specify ax for volume plot.")
        else:
            mplf.plot(
                ts.df,
                ax=ax,
                volume=vol_ax,
                type="candle",
                style="yahoo",
                show_nontrading=True,
            )
    elif style == "ohlc":
        mplf.plot(
            ts.df, ax=ax, type="ohlc", style="mike", show_nontrading=True
        )
    else:
        raise ValueError("Must specify style.")
    return


def plot_moving_averages(ts, ax, windows=None):
    """
    Plot two simple moving averages.
    Default windows are 12 and 26 days. Can be customized by using
    lists of window-color pairs.

    Args:
        ts (:class:`surfingcrypto.ts.TS`) : `surfingcrypto.ts.TS` object
        windows (obj:`list` of obj:`int`,optional): list of
            windows to compute MA
    """

    colors = ["yellow", "orange"]
    if windows is None:
        windows = [ts.ta_params["sma"]["fast"], ts.ta_params["sma"]["slow"]]
    for window, color in zip(windows, colors):
        ax.plot(
            ts.df["SMA_" + str(window)],
            linestyle="-",
            color=color,
            label="SMA" + str(window),
            alpha=0.3,
            linewidth=1,
        )
        l = ax.legend(loc="upper left", prop={"size": 3})
        l.get_frame().set_linewidth(0.5)


def plot_macd(ts, ax, plot_lines=True):
    """
    plot macd indicator into a matplotlib.axes.Axes object.


    Args:
        ts (:class:`surfingcrypto.ts.TS`) : `surfingcrypto.ts.TS` object
        ax (:class:`matplotlib.axes.Axes`) : matplotlib ax to plot MACD into.
        plot_lines (bool): plot lines in addition to histogram.

    """

    fast = str(ts.ta_params["macd"]["fast"])
    slow = str(ts.ta_params["macd"]["slow"])
    sign = str(ts.ta_params["macd"]["signal"])

    prices = ts.df["Open"]
    macd = ts.df["MACD_" + fast + "_" + slow + "_" + sign]
    signal = ts.df["MACDs_" + fast + "_" + slow + "_" + sign]
    hist = ts.df["MACDh_" + fast + "_" + slow + "_" + sign]

    if plot_lines:
        # ax1.plot(prices)
        ax.plot(
            macd,
            color="grey",
            linewidth=1.5,
            label="MACD(" + fast + "-" + slow,
        )
        ax.plot(
            signal,
            color="skyblue",
            linewidth=1.5,
            label="SIGNAL(" + sign + ")",
        )

    for i in range(len(prices)):
        if str(hist[i])[0] == "-":
            ax.bar(prices.index[i], hist[i], color="#ef5350")
        else:
            ax.bar(prices.index[i], hist[i], color="#26a69a")

    if plot_lines:
        l = ax.legend(loc="lower left", prop={"size": 3})
        l.get_frame().set_linewidth(0.5)

    ax.annotate(
        "MACD(" + fast + "-" + slow + ")-S(" + sign + ")",
        xy=(0.99, 0.1),
        xycoords="axes fraction",
        ha="right",
    )
    ax.set_ylabel("MACD")


def plot_bb(ts, iax):
    """
    plot Bollinger bands indicator into a matplotlib.axes.Axes object.

    Args:
        ts (:class:`surfingcrypto.ts.TS`) : `surfingcrypto.ts.TS` object
        ax (:class:`matplotlib.axes.Axes`) : matplotlib ax
            to plot BB bands into.
    """
    length = str(ts.ta_params["bbands"]["length"])
    std = "{0:.1f}".format(ts.ta_params["bbands"]["std"])

    iax.plot(
        ts.df["BBM_" + length + "_" + std],
        label=f"MA{length}-STD{str(std)}",
        color="dodgerblue",
        alpha=0.35,
    )  # middle band
    iax.plot(
        ts.df["BBU_" + length + "_" + std],
        label="_",
        color="greenyellow",
        alpha=0.35,
    )  # Upper band
    iax.plot(
        ts.df["BBL_" + length + "_" + std],
        label="_",
        color="coral",
        alpha=0.35,
    )  # lower band
    iax.fill_between(
        ts.df.index,
        ts.df["BBL_" + length + "_" + std],
        ts.df["BBU_" + length + "_" + std],
        alpha=0.1,
    )
    l = iax.legend(loc="lower left", prop={"size": 3})
    l.get_frame().set_linewidth(0.5)
    iax.set_ylabel("B Bands")


def plot_RSI(ts, iax):
    """
    plot RSI indicator into a matplotlib.axes.Axes object.

    Args:
        ts (:class:`surfingcrypto.ts.TS`) : `surfingcrypto.ts.TS` object
        ax (:class:`matplotlib.axes.Axes`) : matplotlib ax to plot RSI into.
    """
    timeperiod = str(ts.ta_params["rsi"]["timeperiod"])
    iax.plot(ts.df["RSI_" + timeperiod], color="magenta", alpha=0.5)
    iax.set_ylim([0, 100])
    iax.set_ylabel("RSI")
    iax.axhspan(30, 70, facecolor="white", alpha=0.1)
    iax.axhline(30, color="coral", linewidth=0.5, alpha=0.3)
    iax.axhline(70, color="greenyellow", linewidth=0.5, alpha=0.3)


def shiftedColorMap(cmap, start=0, midpoint=0.5, stop=1.0, name="shiftedcmap"):
    """
    Function to offset the "center" of a colormap. Useful for
    data with a negative min and positive max and you want the
    middle of the colormap's dynamic range to be at zero.

    Args:
        cmap : The matplotlib colormap to be altered
        start : Offset from lowest point in the colormap's range.
            Defaults to 0.0 (no lower offset). Should be between
            0.0 and `midpoint`.
        midpoint : The new center of the colormap. Defaults to
            0.5 (no shift). Should be between 0.0 and 1.0. In
            general, this should be  1 - vmax / (vmax + abs(vmin))
            For example if your data range from -15.0 to +5.0 and
            you want the center of the colormap at 0.0, `midpoint`
            should be set to  1 - 5/(5 + 15)) or 0.75
        stop : Offset from highest point in the colormap's range.
            Defaults to 1.0 (no upper offset). Should be between
            `midpoint` and 1.0.
    """
    cdict = {"red": [], "green": [], "blue": [], "alpha": []}

    # regular index to compute the colors
    reg_index = np.linspace(start, stop, 257)

    # shifted index to match the data
    shift_index = np.hstack(
        [
            np.linspace(0.0, midpoint, 128, endpoint=False),
            np.linspace(midpoint, 1.0, 129, endpoint=True),
        ]
    )

    for ri, si in zip(reg_index, shift_index):
        r, g, b, a = cmap(ri)

        cdict["red"].append((si, r, r))
        cdict["green"].append((si, g, g))
        cdict["blue"].append((si, b, b))
        cdict["alpha"].append((si, a, a))

    newcmap = matplotlib.colors.LinearSegmentedColormap(name, cdict)
    plt.register_cmap(cmap=newcmap)

    return newcmap



"""
portfolio value traker.
"""
import datetime
import pandas as pd
import numpy as np
import surfingcrypto
from surfingcrypto.ts import TS


# https://towardsdatascience.com/modeling-your-stock-portfolio-performance-with-python-fbba4ef2ef11


class Tracker:
    """Tracker module

    This module tracks the daily portfolio value,
    plus it features a series of portfolio statistics.

    Args:
        df (pd.DataFrame): _description_
        stocks_start (str, optional): time from which start tracking.
            Defaults to None.
        stocks_end (str, optional): time from which stop tracking.
            Defaults to None.
        configuration (Config, optional): surfingcrypto configuration object.
            Defaults to None.
    """

    def __init__(
        self,
        df: pd.DataFrame,
        stocks_start: str = None,
        stocks_end: str = None,
        benchmark: str = None,
        configuration: surfingcrypto.Config = None,
    ):
        self.configuration = configuration

        # for now, sets time limits to all transactions
        if stocks_start is None:
            self.stocks_start = pd.Timestamp(
                self.portfolio_df["Open date"].min().date(), tz="utc"
            )
        else:
            self.stocks_start = pd.Timestamp(
                datetime.datetime.strptime(stocks_start, "%d-%m-%Y"), tz="utc"
            )

        if stocks_end is None:
            self.stocks_end = pd.Timestamp(
                datetime.datetime.now(datetime.timezone.utc).date()
                + datetime.timedelta(-1),
                tz="utc",
            )
        else:
            self.stocks_end = pd.Timestamp(
                datetime.datetime.strptime(stocks_end, "%d-%m-%Y"), tz="utc"
            )

        self.benchmark = benchmark

        self.portfolio_df = self._format_df(df)
        self.closedata = self._load_data()
        self.active_positions = self._portfolio_start_balance()
        self.daily_snapshots = self._time_fill(self.active_positions)

        if self.benchmark is not None:
            self._set_benchmark()

        self.daily_calcs=self._per_day_portfolio_calcs()

    def _format_df(
        self, df: pd.DataFrame,
    ):
        """
        sets dataframe to required format by traker module.

        Arrguments:
            df (:obj:`pd.DataFrame`):

        Returns:
            portfolio_df (:obj:`pd.DataFrame`): dataframe
                in required format
        """
        portfolio_df = (
            df.reset_index()
            .copy()
            .rename(
                {
                    "datetime": "Open date",
                    "type": "Type",
                    "symbol": "Symbol",
                    "amount": "Qty",
                    "spot_price": "Adj cost per share",
                    "native_amount": "Adj cost",
                },
                axis=1,
            )
        )
        portfolio_df["Open date"] = pd.to_datetime(portfolio_df["Open date"])

        # drop unused columns
        portfolio_df.drop(
            ["trade_id", "nat_symbol", "total", "subtotal", "total_fee"],
            axis=1,
            inplace=True,
        )

        return portfolio_df

    def _load_data(self) -> pd.DataFrame:
        """loads required data from configuration object

        Returns:
            pd.DataFrame: dataframe of close prices
                requested by transactions

        """
        self.error_log = []
        dfs = []

        symbols = self.portfolio_df.Symbol.unique()

        for symbol in symbols:
            try:
                ts = TS(configuration=self.configuration, coin=symbol)

                # considering rebrandings
                rebrandings = [
                    k
                    for k, v in self.configuration.rebrandings.items()
                    if v == ts.coin
                ]
                if rebrandings:
                    ts.coin = rebrandings[0]

                df = self._check_data(
                    ts.df[["Close"]].copy(),
                    pd.Timestamp(
                        self.configuration.coinbase_req[ts.coin]["start"]
                    ),
                    pd.Timestamp(
                        self.configuration.coinbase_req[ts.coin]["end_day"]
                    ),
                )
                df["symbol"] = symbol
                dfs.append(df)

            except Exception as e:
                self.error_log.append({"symbol": symbol, "error": e})

        closedata = pd.concat(dfs, sort=True)
        closedata.reset_index(inplace=True)
        return closedata

    def _set_benchmark(self) -> pd.DataFrame:
        """sets benchmark
        """
        ts = TS(configuration=self.configuration, coin=self.benchmark)
        self.benchmark_df = self._check_data(
            ts.df[["Close"]].copy(), self.stocks_start, self.stocks_end
        )

    def _check_data(
        self, df, i: pd.Timestamp, o: pd.Timestamp,
    ) -> pd.DataFrame:
        if df.index.min() <= i and df.index.max() >= o:
            return df
        else:
            raise ValueError("Local data is not sufficient for purpose.")

    def _position_adjust(
        self, daily_positions: pd.DataFrame, sale: pd.Series
    ) -> pd.DataFrame:
        """adjust daily positions based for given sale

        Given a sale, it adjusts the daily_poisitions dataframe accordingly

        For every buy in buys:
        - If quantity of the oldest buy amount is ≤ the sold quantity
            subtract the amount of the buy position from the sell,
            then set the buy quantity to 0.
        - Else, subtract the sales quantity from the buy position
            subtract that same amount from the sales position

        Args:
            daily_positions (pd.DataFrame): dataframe of positions
            sale (pd.Series): pandas series of sale

        Returns:
            pd.DataFrame: _descr_
        """
        # sorting by ‘Open Date’ to subtract positions using the _fifo method
        stocks_with_sales = pd.DataFrame()
        buys = daily_positions[daily_positions["Type"] == "buy"].sort_values(
            by="Open date"
        )

        # apply correction
        for position in buys[buys["Symbol"] == sale[1]["Symbol"]].iterrows():
            if position[1]["Qty"] <= sale[1]["Qty"]:
                sale[1]["Qty"] -= position[1]["Qty"]
                position[1]["Qty"] = 0
            else:
                position[1]["Qty"] -= sale[1]["Qty"]
                sale[1]["Qty"] -= sale[1]["Qty"]
            stocks_with_sales = stocks_with_sales.append(position[1])
        return stocks_with_sales

    def _portfolio_start_balance(self) -> pd.DataFrame:
        """gets the portfolio start balance

        Gets the portfolio at the given start date,
        updated to its current value by applying
        the `poisition_adjust` method on the positions before
        the start date.

        Returns:
            pd.DataFrame: transactions adjusted to start date
        """
        # create a dataframe of all trades that happened before our start date.
        positions_before_start = self.portfolio_df[
            self.portfolio_df["Open date"] < self.stocks_start
        ]

        # future sales after the start_date since
        future_positions = self.portfolio_df[
            (self.portfolio_df["Open date"] >= self.stocks_start)
        ]

        # get sales of positions before start
        sales = (
            positions_before_start[positions_before_start["Type"] == "sell"]
            .groupby(["Symbol"])["Qty"]
            .sum()
        )
        sales = sales.reset_index()

        # poisitions that didnt change
        positions_no_change = positions_before_start[
            ~positions_before_start["Symbol"].isin(sales["Symbol"].unique())
        ]

        # loop thru the sales adjusting poisitions accordingly
        adj_positions_df = pd.DataFrame()
        for sale in sales.iterrows():
            adj_positions_df = adj_positions_df.append(
                self._position_adjust(positions_before_start, sale)
            )

        # append poisitions and future
        adj_positions_df = adj_positions_df.append(positions_no_change)
        adj_positions_df = adj_positions_df.append(future_positions)

        # filtering out any rows that _position_adjust zeroed out
        adj_positions_df = adj_positions_df[adj_positions_df["Qty"] > 0]

        return adj_positions_df

    def _fifo(self, portfolio, sales, date):

        # Our _fifo function takes your active portfolio positions, the sales
        # dataframe created in _time_fill, and the current date in the
        #  market_cal list.
        # It then filters sales to find any that have occurred on the current
        #  date, and
        # create a dataframe of positions not affected by sales:

        sales = sales[sales["Open date"].dt.date == date]
        daily_positions = portfolio[portfolio["Open date"].dt.date <= date]
        positions_no_change = daily_positions[
            ~daily_positions["Symbol"].isin(sales["Symbol"].unique())
        ]

        # bringing along all future positions
        future_positions = portfolio[portfolio["Open date"].dt.date > date]

        # We’ll then use our trusty _position_adjust function to zero-out any
        #  positions with active sales. If there were no sales for the
        # specific date,
        #   our function will simply append the positions_no_change
        # onto the empty
        #    adj_positions dataframe, leaving you with an accurate
        # daily snapshot of positions:

        adj_positions = pd.DataFrame()
        for sale in sales.iterrows():
            adj_positions = adj_positions.append(
                self._position_adjust(daily_positions, sale)
            )
        adj_positions = adj_positions.append(positions_no_change)
        adj_positions = adj_positions.append(future_positions)
        adj_positions = adj_positions[~np.isclose(adj_positions["Qty"], 0)]
        # adj_positions = adj_positions[adj_positions["Qty"] > 0]
        return adj_positions

    def _time_fill(self, active_df: pd.DataFrame) -> list:
        """adjust active positions of selected time interval

        Providing the dataframe of active positions,
        find the sales, and zero-out sales against buy positions.

        To obtain the positions for each day, this will loop for every day in
        the selected time interval.

        Args:
            active_df (pd.DataFrame) : _descr_

        Returns:
            per_day_balance (:obj:`list` of :obj:`pandas.DataFrame`):
                list of dfs
        """

        portfolio = active_df

        # calendar
        calendar = pd.date_range(
            start=self.stocks_start, end=self.stocks_end, freq="1d"
        )

        # get sales
        sales = (
            portfolio[portfolio["Type"] == "sell"]
            .groupby(["Symbol", pd.Grouper(key="Open date", freq="D")])["Qty"]
            .sum()
        )
        sales = sales.reset_index()

        # sets
        per_day_balance = []
        for date in calendar:
            if (sales["Open date"].dt.date == date).any():
                portfolio = self._fifo(portfolio, sales, date)
            daily_positions = portfolio[portfolio["Open date"].dt.date <= date]
            daily_positions = daily_positions[daily_positions["Type"] == "buy"]
            daily_positions["Date Snapshot"] = date
            per_day_balance.append(daily_positions)
        return per_day_balance

    def _per_day_portfolio_calcs(self) -> pd.DataFrame:
        """calculates daily portfolio stats.

        Calculates
            - Modified cost per share
            - Benchmark calculations

        Returns:
            pd.DataFrame: _description_
        """
        df = pd.concat(self.daily_snapshots)

        # modified cost per share
        df = self._modified_cost_per_share(df)

        # benchmark stats
        if self.benchmark is not None:
            df = self._benchmark_portfolio_calcs(df)

        # calc return
        df = self._calc_returns(df)
        
        return df

    def _modified_cost_per_share(self, portfolio: pd.DataFrame) -> pd.DataFrame:
        """
        matches prices of each asset to open date

        Args:
            portfolio (pd.DataFrame): _description_

        Returns:
            pd.DataFrame: _description_
        """
        df = pd.merge(
            portfolio,
            self.closedata,
            left_on=["Date Snapshot", "Symbol"],
            right_on=["Date", "symbol"],
            how="left",
        )
        df.rename(columns={"Close": "Symbol Adj Close"}, inplace=True)
        df["Adj cost daily"] = df["Symbol Adj Close"] * df["Qty"]
        df = df.drop(["symbol", "Date"], axis=1)
        return df

    # merge portfolio data with latest benchmark data and create several calcs
    def _benchmark_portfolio_calcs(self, portfolio:pd.DataFrame)->pd.DataFrame:
        portfolio = pd.merge(
            portfolio,
            self.benchmark_df,
            left_on=["Date Snapshot"],
            right_on=["Date"],
            how="left",
        )
        portfolio.rename(columns={"Close": "Benchmark Close"}, inplace=True)

        portfolio["Open date_temp"] = portfolio["Open date"].dt.floor("d")

        portfolio = pd.merge(
            portfolio,
            self.benchmark_df,
            left_on=["Open date_temp"],
            right_on=["Date"],
            how="left",
        )
        portfolio.drop("Open date_temp", axis=1, inplace=True)
        portfolio.rename(
            columns={"Close": "Benchmark DayOfBuy Close"}, inplace=True
        )
        portfolio["Benchmark Equiv Shares"] = (
            portfolio["Adj cost"] / portfolio["Benchmark DayOfBuy Close"]
        )
        portfolio["Benchmark Adj Cost Daily"] = (
            portfolio["Benchmark Equiv Shares"] * portfolio["Benchmark Close"]
        )
        return portfolio

    def _calc_returns(self, portfolio: pd.DataFrame) -> pd.DataFrame:
        """calculate daily returns

        Args:
            portfolio (pd.DataFrame): _description_

        Returns:
            pd.DataFrame: _description_
        """
        # symbol
        portfolio["symbol Return"] = (
            portfolio["Symbol Adj Close"] / portfolio["Adj cost per share"] - 1
        )
        portfolio["Stock Gain / (Loss)"] = (
            portfolio["Adj cost daily"]
            - portfolio["Qty"] * portfolio["Adj cost per share"]
        )
        # #benchmark
        portfolio["Benchmark Return"] = (
            portfolio["Benchmark Close"]
            / portfolio["Benchmark DayOfBuy Close"]
            - 1
        )

        portfolio["Benchmark Gain / (Loss)"] = (
            portfolio["Benchmark Adj Cost Daily"] - portfolio["Adj cost"]
        )

        # #others
        # portfolio["Abs Value Compare"] = (
        #     portfolio["Adj cost daily"]
        #     - portfolio["Benchmark DayOfBuy Close"]
        # )
        # portfolio["Abs Value Return"] = (
        #     portfolio["Abs Value Compare"] /
        #   portfolio["Benchmark DayOfBuy Close"]
        # )
        # portfolio["Abs Return Compare"] = (
        #     portfolio["symbol Return"] - portfolio["Benchmark Return"]
        # )
        return portfolio

    def daily_grouped_metrics(
        self, cols: list, by_symbol=False,
    ) -> pd.DataFrame:
        idf = self.daily_calcs.copy()
        idf["Date Snapshot"] = idf["Date Snapshot"].dt.date

        if not by_symbol:
            by = ["Date Snapshot"]
        else:
            by = ["Date Snapshot", "Symbol"]

        # group by day
        grouped_metrics = idf.groupby(by)[cols].sum().reset_index()
        grouped_metrics = pd.melt(
            grouped_metrics, id_vars=by, value_vars=cols,
        )
        grouped_metrics = (
            grouped_metrics.set_index(by + ["variable"])
            .unstack([x for x in by if x != "Date Snapshot"] + ["variable"])
            .sort_index(axis=1, level=1)
        )
        # eventually fill gaps
        grouped_metrics.reindex(
            pd.date_range(
                grouped_metrics.index.min(),
                grouped_metrics.index.max(),
                freq="D",
            )
        )
        return grouped_metrics



"""
portfolio objects.
"""
from surfingcrypto.portfolio import MyCoinbase


class Portfolio:
    """
    User portfolio.

    At the moment the only supported :obj:`portfolio_type` is Coinbase, via
    the :obj:`surfingcrypto.portfolio.coinbase` submodule.

    Has methods for assessing portfolio values, grouping transactions by type,
    calculate total fees, etc.

    With `Portfolio.Analysis._standardize()` it is possible to adapt
    the `surfingcrypto.coinbase.MyCoinbase` object to a standard form
    where there are only `buy` or `sell` trades
    and the amounts are always positive.

    This standard portfolio allows to reconstruct easily the timeline
    of the portfolio value, at each given instant in time.

    Arguments:

    Attributes:
        portfolio_type (string): type of portfolio
        df (:obj:`pandas.DataFrame`): coinbase transaction
            history as from the `surfingcrypto.coinbase.MyCoinbase` output
        std_df (:obj:`pandas.DataFrame`): standardized transaction history,
             contains only `buy` or `sell`
    """

    def __init__(self, portfolio_type, **kwargs):
        self.portfolio_type = portfolio_type
        if self.portfolio_type.lower() == "coinbase":
            self.coinbase = MyCoinbase(active_accounts=False, **kwargs)
            self.coinbase.get_history()
            self._standardize()
            self._init_log()
        else:
            raise NotImplementedError

    def _standardize(self):
        """
        creates a dataframe of only `buy` and `sell` orders,
        for easier portfolio analysis.

        It calculates missing fee information of `trade` transactions.

        Note:
            At the moment does not include `send`,`deposit`
            or `withdrawal` transactions

        """
        self.std_df = self.coinbase.history.df.copy()

        # exclude fiat deposit and withdrawals AND SEND
        self.std_df = self.std_df[
            self.std_df["type"].isin(["buy", "sell", "trade", "send"])
        ]

        # set trades as buy or sell transactions
        m = (self.std_df["type"] == "trade") & (self.std_df["amount"] < 0)
        self.std_df.loc[m, "type"] = "sell"
        m = (self.std_df["type"] == "trade") & (self.std_df["amount"] > 0)
        self.std_df.loc[m, "type"] = "buy"

        # set sends as buy and sells
        m = (self.std_df["type"] == "send") & (self.std_df["amount"] < 0)
        self.std_df.loc[m, "type"] = "sell"
        m = (self.std_df["type"] == "send") & (self.std_df["amount"] > 0)
        self.std_df.loc[m, "type"] = "buy"

        # make all amounts positive
        self.std_df["amount"] = self.std_df["amount"].abs()
        self.std_df["native_amount"] = self.std_df["native_amount"].abs()

        # exclude double transactions coin-fiat for the tracking purpose.
        self.std_df = self.std_df[~(self.std_df["symbol"] == "EUR")]

        #split trades fees among the two
        for trade in self.std_df.trade_id.unique():
            if trade is not None:
                t = self.std_df[self.std_df["trade_id"] == trade]
                if len(t) == 2:
                    fee = t["native_amount"].diff()
                    self.std_df.loc[
                        (self.std_df["trade_id"] == trade), "total_fee"
                    ] = (fee[-1] / 2)

                elif len(t) > 2:
                    raise ValueError("More than 2 trades found.")
                else:
                    raise ValueError(
                        "Did not find 2 trades with matching trade_id"
                    )

    def _init_log(self):
        """
        prints log of initialization status.
        """
        print("####### PORTFOLIO ")
        print("")
        print(self.coinbase)
        print("")

        if len(self.std_df) != len(self.coinbase.history.df):
            n = len(self.coinbase.history.df) - len(self.std_df)
            print("")
            print(
                f"Warning! There are {n} transactions"
                "that were EXCLUDED in std_df."
            )
        if not self.coinbase.history.executed_without_errors():
            print("")
            print("Warning! Errors while handling transactions:")
            print(self.coinbase.history)

    def total_fees(self):
        """
        total fees paid for handled transactions.
        """
        return self.std_df.total_fee.sum()



"""
time-series objects for cryptocurrencies.
"""
import pandas as pd
import os
import pandas_ta as ta


class TS:
    """
    This is an time-series oriented crypto price data object.

    Note:
        Data can be downloaded calling ```surfingcrypto.scraper.Scraper```
        object.

    Arguments:
        configuration (:obj:`surfingcrypto.config.config`): configuration
            object
        coin (str): string representing the crypto coin of choice,
            eg. BTC,ETH

    Attributes:
        df (:obj:`pandas.DataFrame`): dataframe with datetime index
            of ohlc data. Could store also TA indicators if these are
            computed invoking the relative method.
        ta_params (dict): dictionary containing TA parametrization
    """

    def __init__(self, configuration, coin=None):

        self.config = configuration

        if coin is None:
            raise ValueError("Must specify coin.")
        else:
            # rebrandings
            if coin in self.config.rebrandings:
                coin = self.config.rebrandings[coin]
            self.coin = coin
            self.build_ts()

    def build_ts(self):
        """
        reads the data from data stored locally in `data/ts/` and
        saved in .csv format.
        """
        if os.path.isfile(
            self.config.data_folder + "/ts/" + self.coin + ".csv"
        ):
            self.df = pd.read_csv(
                self.config.data_folder + "/ts/" + self.coin + ".csv"
            )
            self.df["Date"] = pd.to_datetime(self.df["Date"], utc=True)
            self.df.set_index("Date", inplace=True)
            if any(self.df.index.duplicated()):
                raise ValueError("Data has duplicates.")
        else:
            raise FileNotFoundError(f"{self.coin}.csv not found.")

    def percentage_diff(self, window=7):
        """
        Percentage difference given a window size.

        Arguments:
            window (int): number of days used to computer percentage
                difference.
        """
        return (
            (self.df.Close[-1] - self.df.Close[-window - 1])
            / (self.df.Close[-window - 1])
            * 100
        )

    def report_percentage_diff(self, windows=[1, 3, 7, 14, 60]):
        """
        Produces verbose and pretty report on latest price
        difference from a given list of windows.

        Arguments:
            windows (:obj:`list` of :obj:`int`): list of windows
                to compute percentage difference.

        """
        s = f"**{self.coin}**\n"
        for window in windows:
            s = (
                s
                + f"- {window}d: "
                + "{:.2f}".format(self.percentage_diff(window))
                + " %\n"
            )
        return s

    # TA INDICATORS SAVED TO DF
    def ta_indicators(self):
        """
        computes the selected TA indicators and appends them to df attribute.
        """
        self.parametrization()

        self.df.ta.sma(length=self.ta_params["sma"]["slow"], append=True)
        self.df.ta.sma(length=self.ta_params["sma"]["fast"], append=True)
        self.df.ta.macd(
            window_slow=self.ta_params["macd"]["slow"],
            window_fast=self.ta_params["macd"]["fast"],
            window_sign=self.ta_params["macd"]["signal"],
            append=True,
        )
        self.df.ta.bbands(
            length=self.ta_params["bbands"]["length"],
            std=self.ta_params["bbands"]["std"],
            append=True,
        )
        self.df.ta.rsi(
            timeperiod=self.ta_params["rsi"]["timeperiod"], append=True
        )

    def parametrization(self):
        """
        sets the default parameters if not specified in config.json file.
        """
        # TA parameters
        if self.coin not in self.config.coins.keys() or self.config.coins[self.coin] == "":
            # default if empty or not specified in coins
            self.ta_params = {
                "sma": {"fast": 12, "slow": 26},
                "macd": {"fast": 12, "slow": 26, "signal": 9},
                "bbands": {"length": 20, "std": 2},
                "rsi": {"timeperiod": 14},
            }
        elif isinstance(self.config.coins[self.coin], dict):
            self.ta_params = self.config.coins[self.coin]
        else:
            raise ValueError(
                "Must provide TA parametrization in the correct format."
            )

    def distance_from_ath(self):
        d = {}
        for idx in self.df.index:
            d[idx] = abs(
                self.df.loc[idx, "Close"] - self.df[:idx]["Close"].max()
            )
        self.df["distance_ATH"] = pd.Series(d)



"""
scraping price data from the internet.
"""
from cryptocmd import CmcScraper
import datetime
import pandas as pd
import os


class Scraper:
    """
    gets all data required by module configuration.

    Arguments:
        configuration (:obj:`surfingcrypto.config.config`): package
            configuration object
        fiat (str): prices will be restituted in the selected fiat,
            defaults is `EUR`
        verbose (bool): wether to log at the end of execution

    Attributes:
        config (:obj:`surfingcrypto.config.config`): package
            configuration object
        fiat (str): fiat symbol
        verbose (bool): wether to print log
        runs (:obj:`list` of :obj:`surfincrypto.scraper.CoinScraper`): list
            of `CoinScraper` objects
        errors (:obj:`list` of :obj:`surfincrypto.scraper.CoinScraper`): list
            of `CoinScraper` objects that have errors
        output (bool): Overall boolean output of process. If
            everything went well.
        output_descrition (str): Overall string description of output.


    """

    def __init__(self, configuration, fiat="EUR", verbose=True):
        self.config = configuration
        self.fiat = fiat
        self.verbose = verbose

    def run(self):
        """runs the scraping process."""
        self.runs = []
        for key in self.config.scraping_req:

            # dates are utc unaware
            start = self.config.scraping_req[key]["start"].date()
            end_day = self.config.scraping_req[key]["end_day"].date()

            if key in self.config.rebrandings:
                key = self.config.rebrandings[key]

            path = self.config.data_folder + "/ts/" + key + ".csv"
            c = CoinScraper(key, self.fiat, start, end_day, path)
            self.runs.append(c)

        self._log()

    def _log(self):
        """
        produce a log of all executions.
        """
        length = len(self.runs)
        self.errors = []
        for run in self.runs:
            if hasattr(run, "error"):
                self.errors.append(run)

        if len(self.errors) == 0:
            self.output_description = "Update successful."
            self.output = True
        else:
            self.output_description = (
                "Update failed."
                f" There are ({len(self.errors)}/{length}) errors."
            )
            self.output = False

        if self.verbose:
            print(self.output_description)


class CoinScraper:
    """
    wrapper of GitHub repo `cryptocmd`.

    Scrapes data from coinmarketcap.com.

    It scrapes data for the crypto coin specified.
    Saves data in a `*.csv` file
    stored at the provided path.
    Checks if the file exists, if not it is downloaded.
    If it exists, it check last element of datetime index and compares it
    with today's date. If required, it updates the `*.csv` file
    to today\'s date.

    Arguments:
        coin (str): symbol of crypto
        fiat (str): fiat of prices
        start (:obj:`datetime.datetime`):  start day
        end_day (:obj:`datetime.datetime`): end day
        path (str): path to csv file

    Attributes:
        coin (str): symbol of crypto
        fiat (str): fiat of prices
        start (:obj:`datetime.datetime`):  start day
        end_day (:obj:`datetime.datetime`): end day
        path (str): path to csv file
        description (str): verbose description of result
        result (bool): outcome of scraping run
        error (:obj:`Excection`): generic error
    """

    def __init__(self, coin, fiat, start, end_day, path):
        self.coin = coin
        self.fiat = fiat

        # dates are utc unaware
        self.start = start
        self.end_day = end_day

        self.path = path

        self._run()

    def _run(self):

        if os.path.isfile(self.path):
            df, first, last = self._load_csv()

            first = first.date()
            last = last.date()

            if first == self.start and last == self.end_day:
                self.description = f"DF: {self.coin} already up to date."
                self.result = True
            else:
                try:
                    self._scrape_missing_data(first, last, df)
                    self.description = f"DF: {self.coin} successfully updated."
                    self.result = True
                except Exception as e:
                    self.description = f"DF: {self.coin} update failed."
                    self.result = False
                    self.error = e
        else:
            try:
                self._scrape_alltime_data()
                self.description = f"DF: {self.coin} successfully downloaded."
                self.result = True
            except Exception as e:
                self.description = f"DF: {self.coin} download failed."
                self.result = False
                self.error = e

    def _scrape_alltime_data(self):
        """
        scrape all time data.
        """
        start = self.start.strftime("%d-%m-%Y")
        end_day = self.end_day.strftime("%d-%m-%Y")
        scraper = CmcScraper(self.coin, start, end_day, fiat=self.fiat)
        scraped = scraper.get_dataframe()
        scraped.set_index("Date", inplace=True)
        scraped.sort_index(inplace=True)
        scraped.to_csv(self.path)

    def _scrape_missing_data(self, first, last, df):
        """
        scrapes the missing data and concatenates it to existing df.

        Arguments:
            first (:obj:`datetime.datetime`): date parsed as string
                with d-m-Y format of first known price
            last (:obj:`datetime.datetime`): date parsed as string
                with d-m-Y format of last known price
            df (:obj:`pandas.DataFrame`): dataframe of
                locally stored data
        """
        if first == self.start and last < self.end_day:
            scraper = self._append_to_end(last)
            scraped = scraper.get_dataframe()

        elif self.start < first and last == self.end_day:
            scraper = self._append_to_front(first)
            if scraper.rows:
                scraped = scraper.get_dataframe()
            else:
                scraped = pd.DataFrame(columns=["Date"])

        elif self.start < first and last < self.end_day:
            # end
            scraper = self._append_to_end(last)
            scraped_last = scraper.get_dataframe()
            # front
            scraper = self._append_to_front(first)
            # quickfix for when start < first
            # because it was created after
            #  it's actually up to date.
            if scraper.rows:
                scraped_first = scraper.get_dataframe()
            else:
                scraped_first = pd.DataFrame(columns=["Date"])

            # concat
            scraped = pd.concat([scraped_last, scraped_first])

        elif first < self.start and last == self.end_day:
            # i have more than needed, but its fine
            # i add nothing
            scraped = pd.DataFrame()

        else:
            raise NotImplementedError

        scraped.set_index("Date", inplace=True)
        df = pd.concat([df, scraped])
        df.sort_index(inplace=True)
        df.to_csv(self.path)

    def _append_to_front(self, first: datetime) -> CmcScraper:
        start = self.start.strftime("%d-%m-%Y")
        day_before_first = (first - datetime.timedelta(1)).strftime("%d-%m-%Y")
        scraper = CmcScraper(
            self.coin, start, day_before_first, fiat=self.fiat
        )

        return scraper

    def _append_to_end(self, last: datetime) -> CmcScraper:
        day_after_last = (last + datetime.timedelta(1)).strftime("%d-%m-%Y")
        end_day = self.end_day.strftime("%d-%m-%Y")
        scraper = CmcScraper(
            self.coin, day_after_last, end_day, fiat=self.fiat
        )
        return scraper

    def _load_csv(self):
        """
        load csv data from directory specified as `data_folder`
        in package config

        Return:
            df (:obj:`pandas.DataFrame`): dataframe of locally stored data
            first (_type_): date parsed as d-m-Y of first known price
            last (_type_): date parsed as d-m-Y of last known price
        """
        df = pd.read_csv(self.path)
        df["Date"] = pd.to_datetime(df["Date"])
        df.set_index("Date", inplace=True)
        first = pd.to_datetime(df.index.values[0])
        last = pd.to_datetime(df.index.values[-1])
        return df, first, last

    def __str__(self):
        start = self.start.strftime("%d-%m-%Y")
        endday = self.end_day.strftime("%d-%m-%Y")
        error = hasattr(self, "error")
        return (
            f"CoinScraper({self.coin},"
            f" start={start},"
            f" end_day={endday},"
            f" error={error},"
            ")"
        )

    def __repr__(self):
        start = self.start.strftime("%d-%m-%Y")
        endday = self.end_day.strftime("%d-%m-%Y")
        error = hasattr(self, "error")
        return (
            f"CoinScraper({self.coin},"
            f" start={start},"
            f" end_day={endday},"
            f" error={error},"
            ")"
        )




from setuptools import setup, find_packages

# read the contents of your README file
from pathlib import Path

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name="surfingcrypto",
    version="0.1.3",
    description="Customizable interface to cryptocurrencies.",
    long_description_content_type="text/markdown",
    long_description=long_description,
    url="https://github.com/giocaizzi/surfingcrypto",
    author="giocaizzi",
    author_email="giocaizzi@gmail.com",
    packages=find_packages(include=["surfingcrypto", "surfingcrypto.*"]),
    setup_requires=[],
    tests_require=[],
    install_requires=[
        "matplotlib",
        "mplfinance",
        "numpy",
        "pandas",
        "coinbase",
        "pandas_ta",
        "python_dateutil",
        "pytrends",
        "trendln",
        "cryptocmd",
    ],
    extras_require={"dev": [],},
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        ],
    project_urls={
        "Documentation": "https://giocaizzi.github.io/surfingcrypto/",
        "Bug Reports": "https://github.com/giocaizzi/surfingcrypto/issues",
        "Source": "https://github.com/giocaizzi/surfingcrypto",
     },
)



"""
package configuration
"""
import json
import os
import pathlib
import datetime, pytz
import dateutil


class Config:
    """
    Class for the package configuration.
    Contains API keys and user-specified parametrization of execution.

    Note:
        `data_folder` is optional. If not specified checks if there
        is a data directory in the parent directory.
        If not, it will be created.

    Arguments:
        config_folder (str): ABSOLUTE path to config folder.
        data_folder (str,optional) : ABSOLUTE path to data folder

    Attributes:
        coinbase (dict): coinbase user configuration
        coinbase_req (:obj:`dict` of :obj:`dict`) dictionary
            containing coinbase requirements
        coins (dict): coins user configuration
        config_folder (str): ABSOLUTE path to config folder.
        data_folder (str) : ABSOLUTE path to data folder
        error_log (:obj:`list`): list of errors
        rebrandings (dict): dictionary of known rebrandings
        scraping_req (:obj:`dict` of :obj:`dict`)
            dictionary containing scraping params
        telegram (dict): telegram user configuration
        temp_folder (str,optional) : ABSOLUTE path to data folder
    """

    def __init__(self, config_folder, data_folder=None):
        self.config_folder = config_folder
        self._set_attributes()
        self._set_data_folder(data_folder)
        self._temp_dir()

        self.rebrandings = {"CGLD": "CELO"}

        # ERROR LOG
        self.error_log = []
        # DATA REQUIREMENTS
        self._set_requirements()

    def _set_requirements(self):
        """
        sets data requirements for scraping module.
        """
        self._read_coinbase_requirements()
        self._format_coinbase_req()
        self._set_scraping_parameters()

    def _set_attributes(self):
        """
        sets attributes based on what is specified in the config.json file.

        """
        # configuration folder
        if os.path.isdir(self.config_folder):
            if os.path.isfile(self.config_folder + "/config.json"):
                with open(self.config_folder + "/config.json", "r") as f:
                    dictionary = json.load(f)
                    for key in dictionary:
                        setattr(self, key, dictionary[key])
            else:
                raise FileNotFoundError(
                    "Configuration file `config.json` not found."
                )
        else:
            raise FileNotFoundError("Configuration folder not found.")

    def _set_data_folder(self, data_folder):
        """
        sets the directory to the data folder.

        Arguments:
            data_folder (str): path
        """
        # HANDLING DATA FOLDER
        if data_folder is None:
            self.data_folder = str(
                (pathlib.Path(self.config_folder).parent).joinpath("data")
            )
            self._make_data_directories()
        else:
            if os.path.isdir(data_folder):
                self.data_folder = data_folder
                self._make_data_directories()
            else:
                raise FileNotFoundError(
                    f"Data folder not found. \n {data_folder}"
                )

    def _make_data_directories(self):
        """
        create data subdirectory structure.
        """
        # data folder
        if not os.path.isdir(self.data_folder):
            os.mkdir(self.data_folder)
        # data/ts subfolder
        if not os.path.isdir(self.data_folder + "/ts"):
            os.mkdir(self.data_folder + "/ts")

    def _temp_dir(self):
        """
        Create temp directory for temporary storing plots to be sent.
        If alreay exist, empty folder.
        """
        # data/temp
        self.temp_folder = self.data_folder + "/temp"
        if not os.path.isdir(self.temp_folder):
            os.mkdir(self.temp_folder)
        else:
            for f in os.listdir(self.temp_folder):
                os.remove(self.temp_folder + "/" + f)

    def _read_coinbase_requirements(self):
        """
        gets the requirements for coinbase portfolio tracking.
        """
        if os.path.isfile(self.config_folder + "/coinbase_accounts.json"):
            with open(
                self.config_folder + "/coinbase_accounts.json", "rb"
            ) as f:
                self.coinbase_req = json.load(f)
        else:
            self.coinbase_req = None

    def _format_coinbase_req(self):
        """
        formats coinbase requirements parameteters to datetime
        """
        req = {}
        # first get - if possible - coinbase requirements
        if self.coinbase_req is not None:
            for account in self.coinbase_req["accounts"]:
                if account["currency"] not in ["EUR"]:
                    # active account
                    if float(account["balance"]) > 0.0:
                        start = dateutil.parser.parse(
                            account["timerange"]["1"],
                        ).replace(hour=0, minute=0, second=0, microsecond=0)
                        req[account["currency"]] = {
                            "start": start,
                            # timedelta is because today's close
                            # isnt yet realized
                            "end_day": (
                                datetime.datetime.now(datetime.timezone.utc)
                                + datetime.timedelta(-1)
                            ).replace(
                                hour=0, minute=0, second=0, microsecond=0
                            ),
                        }
                    # historic account
                    else:
                        req[account["currency"]] = {
                            "start": dateutil.parser.parse(
                                account["timerange"]["1"]
                            ).replace(
                                hour=0, minute=0, second=0, microsecond=0
                            ),
                            "end_day": dateutil.parser.parse(
                                account["timerange"]["0"]
                            ).replace(
                                hour=0, minute=0, second=0, microsecond=0
                            ),
                        }
            # store coinbase requirements paresed correctly
            # for portfolio tracker features
            self.coinbase_req = req

    def _set_scraping_parameters(self):
        """
        sets the parameteters for the `surfingcrypto.Scraper` module
        """
        if self.coinbase_req is not None:
            params = self.coinbase_req.copy()
        else:
            params = {}
        # then, overrun with the reporting requirements
        for coin in self.coins:
            params[coin] = {
                # first date from BTC history to be "relevant", if other coin means first
                # available
                "start": datetime.datetime(
                    2017, 10, 1, tzinfo=datetime.timezone.utc
                ),
                # timedelta is because today's close isnt yet realized
                "end_day": datetime.datetime.now(
                    datetime.timezone.utc
                ).replace(hour=0, minute=0, second=0, microsecond=0)
                + datetime.timedelta(-1),
            }

        self.scraping_req = params

    def add_coins(self, coins: list) -> None:
        """add coins to `coins` attribute

        Args:
            coins (list): list of coin strings
        """
        # add coins to attribute
        for coin in coins:
            # avoid overrunning
            if coin not in self.coins:
                self.coins[coin] = ""
        # rerun
        self._set_scraping_parameters()




"""
get google search trend data
"""
from pytrends.request import TrendReq
import pandas as pd


class GTrends:
    """
    Interface with Google Trends by unofficial API.

    Arguments:
        kw_list (:obj:`list` of :obj:`str`): list of keywords to look for.

    Note:
        Gets data, but the data is "relative".
        1 : day of highest interaction
        0 : day of lowest interaction
    """

    def __init__(self, kw_list=None):
        self.kw_list = kw_list
        pass

    def trends(self):
        df_list = []
        for kw in self.kw_list:
            pytrends = TrendReq(hl="en-US", tz=360)
            pytrends.build_payload(
                [kw], cat=0, timeframe="today 5-y", geo="", gprop=""
            )
            df_list.append(pytrends.interest_over_time())

        df = df_list[0].copy()
        for idf in range(len(df_list) - 1):
            df_list[idf].drop("isPartial", axis=1, inplace=True)
            df = pd.concat([df, df_list[idf + 1]])

        if len(df_list) == 1:
            df.drop("isPartial", axis=1, inplace=True)

        if df.index.duplicated().any():
            df = df.groupby(df.index).sum()

        x2 = pd.date_range(df.index.min(), df.index.max(), freq="1D")
        df2 = df.reindex(x2)
        for column in df2.columns.tolist():
            df2[column] = df2[column].interpolate("time")

        return df2

